# -*- coding: utf-8 -*-
"""
Created on Sat Jan  2 20:54:12 2021

@author: Johannes Neumeyer
"""

from math import sin, cos, radians
from graphics_and_games_klassen import Figur

class Scheinwerfer(Figur):
    """
    Laserscheinwerfer mit buntem Lichtkegel
    """

    def __init__(self, xPos = 100, yPos = 100):
        """
        Scheinwerfer wird erstellt und positioniert
        -- Parameter xPos x-Position des Scheinwerfers
        -- Parameter yPos y-Position des Scheinwerfers
        """
        super().__init__()
        self.PositionSetzen(xPos, yPos)
        self.FigurFestlegen()

    def FigurFestlegen(self):
        """
        Gestaltet die grafische Darstellung des Scheinwerfers.
        """
        self.FigurteilFestlegenEllipse(-50, -50, 100, 100, "schwarz")
        self.FigurteilFestlegenRechteck(-50, -50, 100, 50, "schwarz")
        self.FigurteilFestlegenRechteck(-50, -50, 2, 125, "schwarz")
        self.FigurteilFestlegenRechteck(48, -50, 2, 125, "schwarz")
        self.FigurteilFestlegenRechteck(-50, 75, 100, 2, "schwarz")

    def ScheinwerferEinschalten(self):
        """
        Stellt den Scheinwerfer eingeschaltet (also mit Lichtkegel) dar.
        """
        self.EigeneFigurLoeschen()
        x2 = -sin(radians(2.5)) * 4000
        y2 = -cos(radians(2.5)) * 4000
        x3 = sin(radians(2.5)) * 4000
        y3 = -cos(radians(2.5)) * 4000

        self.FigurteilFestlegenDreieck(0, 0, x2, y2, x3, y3, "gelb")
        self.FigurFestlegen()

    def ScheinwerferAusschalten(self):
        """
        Stellt den Scheinwerfer ausgeschaltet dar.
        """
        self.EigeneFigurLoeschen()
        self.FigurFestlegen()

    def LichtfarbeSetzen(self, farbe):
        """
        Setzt die Lichtfarbe des Scheinwerfers wie angegeben.
        -- Parameter farbe Farbe des Scheinwerfers
        """
        self.EigeneFigurLoeschen()
        x2 = -sin(radians(2.5)) * 4000
        y2 = -cos(radians(2.5)) * 4000
        x3 = sin(radians(2.5)) * 4000
        y3 = -cos(radians(2.5)) * 4000
        self.FigurteilFestlegenDreieck(0, 0, x2, y2, x3, y3, farbe)
        self.FigurFestlegen()

class Buehnenbeleuchtung:
    """
    Beleuchtung einer Buehne mit Scheinwerfern
    """
    def __init__(self):
        """
        Erstellt die Scheinwerfer.
        """
        super().__init__()
        self.scheinwerfer = []
        #  Scheinwerfer erstellen
        index = 0
        while index < 5:
            self.scheinwerfer.append(Scheinwerfer())
            index = index + 1

    def ScheinwerferAufstellen(self):
        """
        Stellt alle Scheinwerfer in einer Reihe auf.
        """
        index = 0
        while index < len(self.scheinwerfer):
            self.scheinwerfer[index].PositionSetzen(index * 65 + 40, 450)
            self.scheinwerfer[index].WinkelSetzen(0)
            index = index + 1

    def AnlageEinschalten(self):
        """
        Alle Scheinwerfer der Anlage werden eingeschalten.
        """
        for s in self.scheinwerfer:
            s.ScheinwerferEinschalten()

b = Buehnenbeleuchtung()
