# -*- coding: utf-8 -*-
"""
Created on Sat Jan  2 12:53:02 2021

@author: Johannes Neumeyer
"""

from random import randint
from graphics_and_games_klassen import Rechteck, Figur, Ereignisbehandlung, Text
from intern.zeichenfenster import Zeichenfenster

class DLRechteck(Rechteck):
    """
    Ein rechteckiges Objekt im Dorianland mit vier unsichtbaren Begrenzungen:
    - linker Rand durch eine schwarze Linie
    - rechter Rand durch eine weisse Linie
    - oberer Rand durch eine hellgruene Linie
    - unterer Rand durch eine hellgelbe Linie
    """

    def __init__(self):
        """
        Initialisiert alle Komponenten des DLRechtecks.
        """
        super().__init__()
        self.randLinks = Rechteck()
        self.randLinks.PositionSetzen(self.x, self.y + 10)
        self.randLinks.GroesseSetzen(1, self.hoehe - 10)
        self.randLinks.FarbeSetzen("schwarz")
        self.randRechts = Rechteck()
        self.randRechts.PositionSetzen(self.x + self.breite - 1, self.y + 10)
        self.randRechts.GroesseSetzen(1, self.hoehe - 10)
        self.randRechts.FarbeSetzen("weiss")
        self.randOben = Rechteck()
        self.randOben.PositionSetzen(self.x + 3, self.y)
        self.randOben.GroesseSetzen(self.breite - 6, 10)
        self.randOben.FarbeSetzen("hellgruen")
        self.randUnten = Rechteck()
        self.randUnten.PositionSetzen(self.x + 3, self.y + self.hoehe - 10)
        self.randUnten.GroesseSetzen(self.breite - 6, 10)
        self.randUnten.FarbeSetzen("hellgelb")
        self.randLinks.GanzNachHintenBringen()
        self.randRechts.GanzNachHintenBringen()
        self.randOben.GanzNachHintenBringen()
        self.randUnten.GanzNachHintenBringen()
        self.koordinaten = Text()
        self.koordinaten.TextGroesseSetzen(10)
        self.KoordinatentextAktualisieren()
        self.koordinaten.PositionSetzen(self.x, self.y)
        self.koordinaten.FarbeSetzen("schwarz")
        self.koordinaten.NachVorneBringen()

    def PositionSetzen(self, xNeu, yNeu):
        """
        Ueberschrieben, da auch die Positionen der unsichtbaren Raender
        und der Koordinatentext aktualisiert werden muessen.
        """
        super().PositionSetzen(xNeu, yNeu)
        self.randLinks.PositionSetzen(self.x, self.y + 10)
        self.randRechts.PositionSetzen(self.x + self.breite - 1, self.y + 10)
        self.randOben.PositionSetzen(self.x + 3, self.y)
        self.randUnten.PositionSetzen(self.x + 3, self.y + self.hoehe - 10)
        self.koordinaten.PositionSetzen(self.x, self.y)
        self.KoordinatentextAktualisieren()

    def GroesseSetzen(self, breiteNeu, hoeheNeu):
        """
        Ueberschrieben, da auch die Groessen/Positionen der unsichtbaren Raender
        aktualisiert werden muessen.
        """
        super().GroesseSetzen(breiteNeu, hoeheNeu)
        self.randLinks.GroesseSetzen(self.randLinks.breite, hoeheNeu - 10)
        self.randRechts.GroesseSetzen(self.randRechts.breite, hoeheNeu - 10)
        self.randRechts.PositionSetzen(self.x + self.breite - 1, self.y + 10)
        self.randOben.GroesseSetzen(breiteNeu - 6, 10)
        self.randUnten.GroesseSetzen(breiteNeu - 6, 10)
        self.randUnten.PositionSetzen(self.x + 3, self.y + self.hoehe - 10)

    def KoordinatentextAktualisieren(self):
        """
        Der Text, der die Koordinaten des Rechtecks angibt, wird aktualisiert.
        """
        self.koordinaten.TextSetzen("(" + str(self.x) + "; " + str(self.y) + ")")


class Stufe(DLRechteck):
    """
    Stufe einer Treppe im Dorianland
    """

    def __init__(self, hoehe):
        """
        Setzt Groesse, Farbe, und y-Position der Stufe
        """
        super().__init__()
        self.GroesseSetzen(25, 25 * hoehe)
        self.FarbeSetzen("blau")
        self.yPos = 400 - 25 * hoehe

    def XPositionSetzen(self, xNeu):
        """
        Positioniert die Stufe in x-Richtung
        """
        self.PositionSetzen(xNeu, self.yPos)

class Stahltraeger(DLRechteck):
    """
    Ein Stahltraeger im Dorianland
    """

    def __init__(self, breite = 200):
        """
        Setzt eine anzugebende Breite, Standardhoehe und die Farbe des Stahltraegers
        """
        super().__init__()
        self.GroesseSetzen(breite, 25)
        self.FarbeSetzen("grau")

class Kiste(DLRechteck):
    """
    Eine Kiste im Dorianland
    """

    def __init__(self):
        """
        Erstellt eine Kiste in der Standardgroesse 1 (50 Pixel)
        """
        super().__init__()
        self.FAKTOR = 50
        self.breiteBrett = self.FAKTOR / 10
        self.KisteAnlegen(1)

    def __init___0(self, groesse):
        """
        Erstellt eine Kiste mit der angegebenen Groesse
        -- Parameter groesse Groesse der Kiste in 50er-Schritten (Kann die Werte 1, 2, 3, ... annehmen.)
        """
        super().__init__()
        self.breiteBrett = groesse * self.faktor / 10
        self.KisteAnlegen(groesse)

    def KisteAnlegen(self, groesse):
        """
        Legt die Kiste an.
        """
        self.groesse = groesse
        self.GroesseSetzen(groesse * self.FAKTOR, groesse * self.FAKTOR)
        self.FarbeSetzen("braun")
        self.brettLO = Rechteck()
        self.brettLO.FarbeSetzen("rot")
        self.brettLO.GroesseSetzen(self.breiteBrett, groesse * self.FAKTOR)
        self.brettLO.PositionSetzen(int((self.x + 0.5 * (groesse * self.FAKTOR) - self.breiteBrett / 2)), self.y)
        self.brettLO.WinkelSetzen(45)
        self.brettLU = Rechteck()
        self.brettLU.FarbeSetzen("rot")
        self.brettLU.PositionSetzen(int((self.x + 0.5 * (groesse * self.FAKTOR) - self.breiteBrett / 2)), self.y)
        self.brettLU.GroesseSetzen(self.breiteBrett, groesse * self.FAKTOR)
        self.brettLU.WinkelSetzen(135)


    def PositionSetzen(self, xNeu, yNeu):
        """
        Setzt die Positionen aller Komponenten der Kiste.
        """
        super().PositionSetzen(xNeu, yNeu)
        self.brettLO.PositionSetzen(int((self.x + 0.5 * (self.groesse * self.FAKTOR) - self.breiteBrett / 2)), self.y)
        self.brettLU.PositionSetzen(int((self.x + 0.5 * (self.groesse * self.FAKTOR) - self.breiteBrett / 2)), self.y)

    def HoeheGeben(self):
        """
        Gibt die Hoehe der Kiste zurueck.
        """
        return self.hoehe

class Ziel(DLRechteck):
    """
    Ziel eines Levels im Dorianland
    """

    def __init__(self):
        """
        Setzt Groesse und Farbe des Ziels
        """
        super().__init__()
        self.GroesseSetzen(50, 10)
        self.FarbeSetzen("cyan")

class Dorian(Figur):
    """
    Spielfigur Dorian
    """

    def __init__(self):
        """
        Setzt den Anfangszustand (u. a. Position, Groesse)
        """
        super().__init__()
        self.stop = False
        self.sprung = False
        self.sprungzaehler = 0
        self.teilsprungzaehler = 0
        self.FigurFestlegen()
        self.PositionSetzen(50, 100)

    def TasteGedrueckt(self, taste):
        """
        Laesst Dorian auf Tastendruecke reagieren.
        """
        # Leitet das Springen von Dorian ein.
        if taste == 'space':
            if not self.sprung:
                self.sprungzaehler = 12
                self.teilsprungzaehler = 5
                self.sprung = True

        # Steuert die Links- und Rechtsbewegung von Dorian.
        if not self.stop:
            if taste == 'right':
                if not self.BeruehrtFarbe("schwarz") and self.XPositionGeben() < 800:
                    self.PositionSetzen(self.XPositionGeben() + 4, self.YPositionGeben())
            elif taste == 'left' and self.XPositionGeben() > 0:
                if not self.BeruehrtFarbe("weiss"):
                    self.PositionSetzen(self.XPositionGeben() - 4, self.YPositionGeben())

    def AktionAusfuehren(self):
        """
        Steuert das Springen und Fallen von Dorian.
        """
        #  nach oben Springen
        if self.sprungzaehler > 0:
            self.Teilsprung()
        #  Fallen
        if not (self.BeruehrtFarbe("hellgruen")) and self.YPositionGeben() < 700 and self.sprungzaehler == 0:
            #  Fallen nach dem Absprung
            if self.sprung:
                self.PositionSetzen(self.XPositionGeben(), self.YPositionGeben() + 1)
                if not self.BeruehrtFarbe("hellgruen") and self.YPositionGeben() < 700:
                    self.PositionSetzen(self.XPositionGeben(), self.YPositionGeben() + 1)
            else:
                #  Fallen ohne vorangegangenen Absprung (senkrecht nach unten fallen)
                while ((not self.BeruehrtFarbe("hellgruen")) and self.YPositionGeben() < 700 and self.sprungzaehler == 0):
                    self.PositionSetzen(self.XPositionGeben(), self.YPositionGeben() + 1)
                    Zeichenfenster().FensterNeuZeichnen()
            Zeichenfenster().FensterNeuZeichnen()
            if self.BeruehrtFarbe("hellgruen"):
                self.sprung = False

    def Teilsprung(self):
        """
        Prueft, ob Dorian beim Springen anstoesst,
        und fuehrt ansonsten die Sprungroutine weiter.
        """
        if self.teilsprungzaehler <= 0:
            #  Wenn Dorian mit dem Kopf an einem DLRechteck anstoesst
            if self.BeruehrtFarbe("hellgelb"):
                self.sprungzaehler = 0
                self.teilsprungzaehler = 0
            else:
                self.PositionSetzen(self.XPositionGeben(), self.YPositionGeben() - 5)
                self.sprungzaehler = self.sprungzaehler - 1
                self.teilsprungzaehler = 5
        else:
            self.teilsprungzaehler = self.teilsprungzaehler - 1

    def Stoppen(self):
        """
        Stoppt das Spiel, z. B. wenn Dorian das Ziel eines Levels erreicht hat
        """
        self.stop = True

    def FigurFestlegen(self):
        """
        Legt das Aussehen von Dorian fest.
        """
        self.GroesseSetzen(50)
        #  Haare
        self.FigurteilFestlegenRechteck(-32, -50, 30, 6, "rot")
        self.FigurteilFestlegenRechteck(-38, -44, 48, 6, "rot")
        # Gesicht
        self.FigurteilFestlegenRechteck(-20, -38, 24, 6, "orange");
        self.FigurteilFestlegenRechteck(-38, -32, 48, 18, "orange");
        self.FigurteilFestlegenRechteck(-32, -14, 40, 6, "orange");
        self.FigurteilFestlegenRechteck(10, -26, 6, 6, "orange");
        self.FigurteilFestlegenRechteck(-38, -38, 18, 6, "rot");
        self.FigurteilFestlegenRechteck(-44, -32, 6, 18, "rot");
        self.FigurteilFestlegenRechteck(-32, -32, 6, 12, "rot");
        self.FigurteilFestlegenRechteck(-38, -20, 6, 6, "rot");
        self.FigurteilFestlegenRechteck(-26, -26, 6, 6, "rot");
        self.FigurteilFestlegenRechteck(-8, -38, 6, 12, "schwarz");
        self.FigurteilFestlegenRechteck(-8, -20, 18, 6, "schwarz");
        # blaues Hemd
        self.FigurteilFestlegenRechteck(-38, -8, 48, 6, "blau");
        self.FigurteilFestlegenRechteck(-44, -2, 60, 6, "blau");
        self.FigurteilFestlegenRechteck(-50, 4, 72, 6, "blau");
        self.FigurteilFestlegenRechteck(-38, 10, 6, 6, "blau");
        self.FigurteilFestlegenRechteck(4, 10, 6, 6, "blau");
        # Hände
        self.FigurteilFestlegenRechteck(-50, 10, 12, 18, "orange");
        self.FigurteilFestlegenRechteck(-38, 16, 6, 6, "orange");
        self.FigurteilFestlegenRechteck(10, 10, 12, 18, "orange");
        self.FigurteilFestlegenRechteck(6, 16, 6, 6, "orange");
        # braune Hose
        self.FigurteilFestlegenRechteck(-20, 10, 12, 14, "braun");
        self.FigurteilFestlegenRechteck(-32, 10, 12, 24, "braun");
        self.FigurteilFestlegenRechteck(-8, 10, 12, 24, "braun");
        self.FigurteilFestlegenRechteck(-38, 22, 6, 12, "braun");
        self.FigurteilFestlegenRechteck(4, 22, 6, 12, "braun");
        # Schuhe
        self.FigurteilFestlegenRechteck(-44, 34, 18, 12, "grau");
        self.FigurteilFestlegenRechteck(-50, 40, 6, 6, "grau");
        self.FigurteilFestlegenRechteck(-2, 34, 18, 12, "grau");
        self.FigurteilFestlegenRechteck(16, 40, 6, 6, "grau");

class Gleiter(Figur):
    """
    Gefaehrliches gleitendes Tier im Dorianland
    """

    def __init__(self):
        """
        Legt die Figur und die Startrichtung fest.
        """
        super().__init__()
        self.richtung = 'r'
        self.FigurFestlegen()

    def AktionAusfuehren(self):
        """
        Steuert die Bewegungen des Gleiters.
        """
        #  Fallen
        if not self.BeruehrtFarbe("grau"):
            while not self.BeruehrtFarbe("grau") and self.YPositionGeben() < 700:
                self.PositionSetzen(self.XPositionGeben(), self.YPositionGeben() + 1)
                Zeichenfenster().FensterNeuZeichnen()
        else:
            #  Hin- und Herkrabbeln
            if self.richtung == 'r':
                if not self.BeruehrtFarbe("schwarz"):
                    self.PositionSetzen(self.XPositionGeben() + 1, self.YPositionGeben())
                else:
                    self.richtung = 'l'
            else:
                if not self.BeruehrtFarbe("weiss"):
                    self.PositionSetzen(self.XPositionGeben() - 1, self.YPositionGeben())
                else:
                    self.richtung = 'r'

    def FigurFestlegen(self):
        """
        Legt das Aussehen des Gleiters fest.
        """
        self.FigurteilFestlegenEllipse(-40, -50, 80, 50, "gruen")
        self.FigurteilFestlegenEllipse(-15, -30, 10, 10, "rot")
        self.FigurteilFestlegenEllipse(5, -30, 10, 10, "rot")
        self.FigurteilFestlegenDreieck(-20, -35, -10, -30, -5, -35, "rot")
        self.FigurteilFestlegenDreieck(20, -35, 10, -30, 5, -35, "rot")
        self.FigurteilFestlegenRechteck(-20, -15, 40, 5, "rot")

class DorianLand(Ereignisbehandlung):
    """
    Jump 'n' Run-Spiel mit dem Protagonisten Dorian
    """

    def __init__(self, levelNr):
        """
        Initialisiert das Spiel mit dem angegebenen Level.
        """
        super().__init__()
        self.dorian = Dorian()
        text = Text()
        text.TextSetzen("← →: Gehen     Leertaste: Springen")
        text.PositionSetzen(300, 500)
        self.LevelSetzen(levelNr)
        self.TaktdauerSetzen(10)
        self.Starten()

    def LevelSetzen(self, levelNr):
        """
        Setzt das gewuenschte Level.
        """
        if levelNr == 1:
            self.Level1()
        elif levelNr == 2:
            self.Level2()
        elif levelNr == 3:
            self.Level3()
        else:
            print("Ein Level mit dieser Nummer existiert nicht!")

    def Level1(self):
        """
        Erstellt die Welt fuer Level 1.
        """
        #  Weg
        weg = Stahltraeger(800)
        weg.PositionSetzen(0, 400)
        #  Kisten und Untier
        k1 = Kiste()
        k1.PositionSetzen(250, 350)
        k2 = Kiste()
        k2.PositionSetzen(450, 350)
        g = Gleiter()
        g.PositionSetzen(375, 375)
        #  Ziel
        ziel = Ziel()
        ziel.PositionSetzen(735, 300)
        #=====================
        # Zu ergaenzender Code
        #=====================
        st = Stahltraeger()
        st.PositionSetzen(275, 325)
        k3 = Kiste()
        k3.PositionSetzen(635, 350)
        k4 = Kiste()
        k4.PositionSetzen(685, 350)
        k5 = Kiste()
        k5.PositionSetzen(685, 300)

    def Level2(self):
        """
        Erstellt die Welt fuer Level 2.
        """
        #  Weg oben
        oben = Stahltraeger(100)
        oben.PositionSetzen(0, 150)
        #  Weg unten
        unten = Stahltraeger(800)
        unten.PositionSetzen(100, 400)
        #  Ziel
        ziel = Ziel()
        ziel.PositionSetzen(735, 275)
        
        #  Treppe1 bauen
        treppe1 = []
        
        #  Treppenstufen erstellen und dem Feld hinzufuegen
        treppe1.append(Stufe(10))
        treppe1.append(Stufe(9))
        treppe1.append(Stufe(8))
        treppe1.append(Stufe(3))
        
        #  Treppenstufen positionieren
        treppe1[0].XPositionSetzen(100)
        treppe1[1].XPositionSetzen(125)
        treppe1[2].XPositionSetzen(150)
        treppe1[3].XPositionSetzen(275)
            
        #  Treppe2 bauen

    def AktionAusfuehren(self):
        """
        Mitverantwortlich fuer die Steuerung von Dorian
        """

        if self.dorian.BeruehrtFarbe("gruen"):
            self.Anhalten()
            self.dorian.Stoppen()
        elif self.dorian.BeruehrtFarbe("cyan"):
            self.Anhalten()
            self.dorian.Stoppen()

d = DorianLand(2)
