# -*- coding: utf-8 -*-
"""
Created on Sat Feb 20 20:36:10 2021

@author: Johannes Neumeyer
"""

from graphics_and_games_klassen import Rechteck, Figur


class XXOKaestchen(Rechteck):
    """
    Kaestchen des 3x3-Spielbretts von XXO (Tic Tac Toe)
    ("Kaestchen" in Abgrenzung zum in diesem Kontext
    mehrdeutigen Begriff "Feld"!)
    """

    def __init__(self):
        """
        Erstellt ein Kaestchen fuer das Spielbrett von XXO
        """
        super().__init__()
        self.FarbeSetzen("schwarz")
        self.GroesseSetzen(50, 50)
        #  oranges Inneres des Kaestchens
        self.innen = Rechteck()
        self.innen.FarbeSetzen("orange")
        self.innen.GroesseSetzen(48, 48)

    def PositionSetzen(self, xNeu, yNeu):
        """
        Setzt die Position des XXO-Kaestchens
        """
        super().PositionSetzen(xNeu, yNeu)
        self.innen.PositionSetzen(self.x + 1, self.y + 1)


class XXO:
    """
    Spielbrett des Spiels XXO (Tic Tac Toe), bestehend aus neun Kaestchen
    """

    def __init__(self):
        """
        Legt das Spielbrett an
        """
        #  Erstellen des "aeusseren" Felds, das die einzelnen Spielbrettzeilen verwalten soll
        self.spielbrett = []

        #------------------------
        # Spielbrett befuellen
        #------------------------
        for zeile in range(0, 3):
            # jeweils eine leere aktuelle Zeile hinzufuegen ...
            self.spielbrett.append([])
            for spalte in range (0, 3):
                # ... und diese mit Kaestchen befuellen
                self.spielbrett[zeile].append(XXOKaestchen())

        #------------------------------
        # Darstellen des Spielbretts
        #------------------------------
        for zeile in range (0, 3):
            for spalte in range (0, 3):
                k = self.spielbrett[zeile][spalte]
                k.PositionSetzen(spalte * 50, zeile * 50)

class Schachkaestchen(Rechteck):
    """
    Kaestchen eines Schachbretts
    ("Kaestchen" in Abgrenzung zum in diesem Kontext
    mehrdeutigen Begriff "Feld"!)
    """
    #  Speichert eine Referenz auf die Figur, die sich auf dem Kaestchen befindet
    #  bzw. die leere Referenz, wenn keine Figur auf dem Kaestchen steht.
    eigeneFigur = None

    def __init__(self):
        """
        Legt ein Schachkaestchen mit den Massen 50 x 50 fest, das anfangs unbesetzt ist.
        """
        super().__init__()
        self.GroesseSetzen(50, 50)
        self.eigeneFigur = None

    def BelegenMit(self, spielfigur):
        """
        Setzt die Referenz der Figur, die das Kaestchen nun besetzt,
        und platziert die Figur auf dem Kaestchen.
        """
        self.eigeneFigur = spielfigur
        spielfigur.PositionSetzen(self.x + 25, self.y + 25)
