# -*- coding: utf-8 -*-
"""
Created on Fry Jan 1 12:33:57 2021

@author: Klaus Reinold
"""

from graphics_and_games_klassen import Rechteck, Kreis, Text, Ereignisbehandlung
   


class Button(Ereignisbehandlung):
    """
    Klasse zur Beschreibung eines Buttons
    """
    
    def __init__(self):
        """
        Initialisierung des Buttons
        """    
        super().__init__()
        self.x = 50
        self.y = 50
        self.breite = 70

        self.farbe = "rot"
        self.rahmen = Rechteck()
        self.rahmen.GroesseSetzen(70, 20)
        self.rahmen.PositionSetzen(self.x, self.y)
        self.rahmen.FarbeSetzen(self.farbe)

        self.text = Text()
        self.text.TextGroesseSetzen(16)
        self.text.PositionSetzen(self.x + 5, self.y + 6)
        self.text.TextSetzen("Klick mich")

    


    def MausGeklickt(self, xKlick, yKlick, anzahl):
        """
        Ereignisroutine des Mausklicks
        """    
        if xKlick >= self.x and xKlick <=self.x + self.breite and yKlick > self.y and yKlick < self.y + 20:
            self.AufMausklickReagieren()


    


    def FarbeSetzen(self, farbeNeu):
        """
        Farbe des Buttons setzen
        -- Parameter farbeNeu Farbe 
        """    
        self.rahmen.FarbeSetzen(farbeNeu)
    


    def TextSetzen(self, textNeu):
        """
        Beschriftung des Buttons setzen
        -- Parameter textNeu Text
        """    
        self.text.TextSetzen(textNeu)
    


    def PositionSetzen(self, xNeu, yNeu):
        """
        Position des Buttons setzen
        -- Parameter xNeu neue x-Koordinate
        -- Parameter yNeu neue y-Koordinate
        """    
        self.x = xNeu
        self.y = yNeu
        self.rahmen.PositionSetzen(self.x, self.y)
        self.text.PositionSetzen(self.x + 5, self.y + 6)
    


    def AufMausklickReagieren(self):
        """
        Methode definiert die Reaktion, falls der Button angeklickt wurde.
        Methode bei Bedarf überschreibennot 
        """    
        if self.farbe == "rot":
            self.farbe = "grün"
        else:
            self.farbe = "rot"      
        self.FarbeSetzen(self.farbe)
    




class Textfeld(Ereignisbehandlung):
    """
    Klasse zur Beschreibung eines interaktiven Textfeldes
    """

    def __init__(self):
        """
        Initialisierung des Textfeldes
        """    
        super().__init__()
        self.x = 10
        self.y = 10
        self.breite = 300
        self.inhalt = "Klick mich an und schreib was rein."
        self.istAngeklickt = False

        self.rahmen = Rechteck()
        self.rahmen.GroesseSetzen(300, 20)
        self.rahmen.PositionSetzen(self.x, self.y)
        self.rahmen.FarbeSetzen("weiss")

        self.text = Text()
        self.text.TextGroesseSetzen(16)
        self.text.PositionSetzen(self.x + 5, self.y + 6)
        self.text.TextSetzen(self.inhalt)

    


    def MausGeklickt(self, xKlick, yKlick, anzahl):
        """
        Ereignisroutine des Mausklicks
        -- Parameter xKlick x-Position des Mausklicks
        -- Parameter yKlick y-Position des Mausklicks
        -- Parameter anzahl Anzahl der Klicks
        """    
        if xKlick >= self.x and xKlick <=self.x + self.breite and yKlick > self.y and yKlick < self.y + 20:
            self.istAngeklickt = True    
        else:
            self.istAngeklickt = False


    def TasteGedrueckt(self, c):
        """
        Ereignisroutine des Tastendrucks
        -- Parameter gedrückte Taste
        """    
        if self.istAngeklickt:
        
            if self.inhalt == "Klick mich an und schreib was rein.":
                self.inhalt = ""+c
            else:
                self.inhalt = self.inhalt + c
            self.text.TextSetzen(self.inhalt)


    def PositionSetzen(self, xNeu, yNeu):
        """
        Position des Textfeldes setzen
        -- Parameter xNeu neue x-Koordinate
        -- Parameter yNeu neue y-Koordinate
        """    
        self.x = xNeu
        self.y = yNeu
        self.rahmen.PositionSetzen(self.x, self.y)
        self.text.PositionSetzen(self.x + 5, self.y+6)
    
    

    def TextSetzen(self, tNeu):
        """
        Methode zum Setzen des Textes
        -- Parameter tneu neuer Text
        """   
        self.inhalt = tNeu
        self.text.TextSetzen(self.inhalt)
    


class Checkbox(Ereignisbehandlung):
    """
    Klasse zur Beschreibung einer Checkbox
    """
    
    def __init__(self):
        """
        Initialisierung der Checkbox
        """    
        super().__init__()
        self.x = 20
        self.y = 20
        self.breite = 300
        self.istAusgewaehlt = False

        self.rahmen = Rechteck()
        self.rahmen.GroesseSetzen(12, 12)
        self.rahmen.FarbeSetzen("weiss")

        self.inneresRechteck = Rechteck()
        self.inneresRechteck.GroesseSetzen(8, 8)
        self.inneresRechteck.FarbeSetzen("schwarz")
        self.inneresRechteck.SichtbarkeitSetzen(False)
        
        self.text = Text()
        self.text.TextGroesseSetzen(16)
        self.text.TextSetzen("Beschreibung der Checkbox")

        self.PositionSetzen(self.x, self.y)


    def MausGeklickt(self, xKlick, yKlick, anzahl):
        """
        Ereignisroutine des Mausklicks
        """    
        if (xKlick - self.x)*(xKlick - self.x) + (yKlick - self.y)*(yKlick - self.y) <= 64:
            if self.istAusgewaehlt:
                self.istAusgewaehlt = False
                self.inneresRechteck.SichtbarkeitSetzen(False)
            else:
                self.istAusgewaehlt = True
                self.inneresRechteck.SichtbarkeitSetzen(True)


    def TextSetzen(self, textNeu):
        """
        Beschriftung des Buttons setzen
        -- Parameter textNeu Text
        """    
        self.text.TextSetzen(textNeu)
    


    def PositionSetzen(self, xNeu, yNeu):
        """
        Position des Buttons setzen
        -- Parameter xNeu neue x-Koordinate
        -- Parameter yNeu neue y-Koordinate
        """    
        self.x = xNeu
        self.y = yNeu
        self.rahmen.PositionSetzen(self.x-6, self.y-6)
        self.inneresRechteck.PositionSetzen(self.x-4, self.y-4)
        self.text.PositionSetzen(self.x + 10, self.y - 6)


class Radiobutton(Ereignisbehandlung):
    """
    Klasse zur Beschreibung eines Radiobuttons
    """
    
    def __init__(self):
        """
        Initialisierung des Radiobuttons
        """    
        super().__init__()
        self.x = 20
        self.y = 20
        self.breite = 300
        self.istAusgewaehlt = False

        self.rahmen = Kreis()
        self.rahmen.RadiusSetzen(6)
        self.rahmen.FarbeSetzen("weiss")

        self.innenkreis = Kreis()
        self.innenkreis.RadiusSetzen(4)
        self.innenkreis.FarbeSetzen("schwarz")
        self.innenkreis.SichtbarkeitSetzen(False)
        
        self.text = Text()
        self.text.TextGroesseSetzen(16)
        self.text.TextSetzen("Beschreibung des Radiobuttons")

        self.PositionSetzen(self.x, self.y)


    def MausGeklickt(self, xKlick, yKlick, anzahl):
        """
        Ereignisroutine des Mausklicks
        """    
        if (xKlick - self.x)*(xKlick - self.x) + (yKlick - self.y)*(yKlick - self.y) <= 64:
            self.istAusgewaehlt = True
            self.innenkreis.SichtbarkeitSetzen(True)



    def TextSetzen(self, textNeu):
        """
        Beschriftung des Radiobuttons setzen
        -- Parameter textNeu Text
        """    
        self.text.TextSetzen(textNeu)
    


    def PositionSetzen(self, xNeu, yNeu):
        """
        Position des Buttons setzen
        -- Parameter xNeu neue x-Koordinate
        -- Parameter yNeu neue y-Koordinate
        """    
        self.x = xNeu
        self.y = yNeu
        self.rahmen.PositionSetzen(self.x, self.y)
        self.innenkreis.PositionSetzen(self.x, self.y)
        self.text.PositionSetzen(self.x + 10, self.y - 6)


class MeinButton(Button):
    """
    Überschreiben der Klasse Button zur Anpassung an die spezifischen
    Anforderungen der GUI
    """    
    def __init__(self, textNeu):
        """
        Initialisierung des MeinButtons
        """    
        super().__init__()
        self.t = textNeu
    
    def AufMausklickReagieren(self):
        self.t.TextSetzen("")


class GUI:
    """
    Beschreiben Sie hier die Klasse GUI.
    """

    def __init__(self):
        """
        Initialisierung der GUI
        """    
        self.t = Textfeld()
        self.b = MeinButton(self.t)
        self.check = Checkbox()
        self.check.PositionSetzen(20, 80)
        self.check.TextSetzen("Schalte mich ein und aus")
        self.radio = Radiobutton()
        self.radio.PositionSetzen(20, 100)
        self.radio.TextSetzen("Schalte mich ein")
    
GUI()