# -*- coding: utf-8 -*-
"""
Created on Sat Feb 20 14:30:46 2021

@author: Johannes Neumeyer
"""

import csv

class Datenpunkt:
    """
    Datenpunkt, bestehend aus einer Zeit in ms und einem Spannungswert in mV
    """

    def __init__(self, zeitpunktNeu, spannungNeu):
        """
        Usereingaben werden gespeichert
        """
        self.zeitpunkt = zeitpunktNeu
        self.spannung = spannungNeu

    def ZeitpunktGeben(self):
        """
        Gibt den Zeitpunkt zurueck
        --return Aufnahmezeit des Datenpunktes
        """
        return self.zeitpunkt

    def SpannungGeben(self):
        """
        Gibt die Spannung zurueck
        --return Spannungswert des Datenpunktes
        """
        return self.spannung

class Dateileser:
    """
    Mit der Klasse Dateileser kann eine einspaltige csv-Datei ausgelesen werden.
    """

    def __init__(self, dateiname):
        self.dateiname = dateiname
        self.daten = []


    def ZahlenAuslesen(self):
        """
        Liest ganzzahlige Werte in das Feld ein und gibt das Feld zurueck.
        """

        with open(self.dateiname) as csvdatei:
            csv_reader_object = csv.reader(csvdatei)

            # row ist jeweils eine Liste von Strings
            for row in csv_reader_object:
                if len(row)>0:
                    self.daten.append(int(row[0]))

        return self.daten

    def ZahlenAuslesenUndAnzeigen(self):
        """
        Liest ganzzahlige Werte in das Feld ein und gibt die Werte aus.
        """
        self.ZahlenAuslesen()

        for d in self.daten:
            print(d)

class EKG_Auswertung:
    """
    Auswertung eines EKGs
    """

    def __init__(self):
        """
        Liest die Datenreihe ein.
        """
        self.dateileser = Dateileser("EKG_Datenreihe1.csv")
        self.daten = self.dateileser.ZahlenAuslesen()
        #  Zeitabstand zwischen zwei Messwerten in Millisekunden
        self.zeitabstand = 2
        self.maxima = []

    def DatenreiheAusgeben(self):
        """
        Ausgabe einer Messreihe mit Zeitpunkt in ms und Spannung in mV
        """
        index = 0
        while index < len(self.daten):
            print(str(index * 2) + " ms:   " + " mV")
            index = index + 1

    def MaximaBestimmen(self):
        """
        Ermittelt die maximalen Spannungswerte innerhalb der QRS-Komplexe
        und deren Zeitpunkte und gibt eine Liste zurueck, die diese Werte als Datenpunkte
        enthaelt.
        --return Feld mit den Daten der Maxima
        """
        maximumSpannung = 0
        zeitpunktMaximum = 0
        anzahlFallendeWerte = 0

        index = 0
        while index < len(self.daten):
            if self.daten[index] > 300:
                #  es werden nur Maxima der QRS-Komplexe gesucht -> Schwellwert muss ueberschritten sein!
                if self.daten[index] > maximumSpannung:
                    maximumSpannung = self.daten[index]
                    zeitpunktMaximum = index * self.zeitabstand
                else:
                    anzahlFallendeWerte = anzahlFallendeWerte + 1
                    # Anzahl der Schritte nach unten werden gezaehlt
                    if anzahlFallendeWerte > 3:
                        #  bei mehr als drei Schritten war es kein Ausreisser -> lokales Maximmum gefunden!
                        self.maxima.append(Datenpunkt(zeitpunktMaximum, maximumSpannung))
                        index = index + 50 # 100 ms nach vorne springen
                        zeitpunktMaximum = 0 # temporaere Variablen zuruecksetzen
                        maximumSpannung = 0 # temporaere Variablen zuruecksetzen
            index = index + 1
        return self.maxima
